package org.apache.pdfbox.util;

import java.awt.Dimension;
import java.awt.geom.AffineTransform;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDResources;
import org.apache.pdfbox.pdmodel.common.PDRectangle;
import org.apache.pdfbox.util.PDFStreamEngine;
import org.apache.pdfbox.util.ResourceLoader;

public class PDFImageDetailsResolver extends PDFStreamEngine {

	public PDFImageDetailsResolver() throws IOException {
		super(ResourceLoader.loadProperties("org/apache/pdfbox/resources/PDFImageDetailsResolver.properties", true));
	}

	private PDPage page;
	private Dimension pageSize;
	private List<ResolvedImage> resolvedImages;
	
	public List<ResolvedImage> findImages(PDPage page) throws IOException {
		this.page=page;
		resolvedImages=new ArrayList<PDFImageDetailsResolver.ResolvedImage>();
		
		PDRectangle cropBox = page.findCropBox();
        float widthPt = cropBox.getWidth();
        float heightPt = cropBox.getHeight();
        
        pageSize=new Dimension( (int)widthPt, (int)heightPt );
		
		if (page.getContents()!=null) {
			PDResources resources=page.findResources();
			processStream(page, resources, page.getContents().getStream());
		}
		return resolvedImages;
	}
	
	public PDPage getPage() {
		return page;
	}
	
	public Dimension getPageSize() {
        return pageSize;
    }

	public void processImage(float imageWIdth, float imageHeight, AffineTransform at, double angle, String name) {
		System.out.println(""+at);
		ResolvedImage resolvedImage=new ResolvedImage();
		resolvedImage.setName(name);
		resolvedImage.setWidth(imageWIdth);
		resolvedImage.setHeight(imageHeight);
		resolvedImage.setX(at.getTranslateX());
		resolvedImage.setY(at.getTranslateY());
		
		resolvedImage.setWidthScale(at.getScaleX());
		resolvedImage.setHeightScale(at.getScaleY());
		
		resolvedImage.setAngle(angle);
		
		resolvedImages.add(resolvedImage);
	}
	
	public class ResolvedImage {
		private String name;
		
		private float width;
		private float height;
		
		private double widthScale;
		private double heightScale;
		
		private double angle;
		
		private double x;
		private double y;
		
		public String getName() {
			return name;
		}
		private void setName(String name) {
			this.name = name;
		}
		public float getWidth() {
			return width;
		}
		private void setWidth(float width) {
			this.width = width;
		}
		public float getHeight() {
			return height;
		}
		
		public float getScaledWidth() {
			return (float)(width*widthScale);
		}
		public float getScaledHeight() {
			return (float)(height*heightScale);
		}
		
		private void setHeight(float height) {
			this.height = height;
		}
		public float getX() {
			return (float)x;
		}
		private void setX(double x) {
			this.x = x;
		}
		public float getY() {
			return (float)y;
		}
		private void setY(double y) {
			this.y = y;
		}
		public double getWidthScale() {
			return widthScale;
		}
		private void setWidthScale(double widthScale) {
			this.widthScale = widthScale;
		}
		public double getHeightScale() {
			return heightScale;
		}
		private void setHeightScale(double heightScale) {
			this.heightScale = heightScale;
		}
		public double getAngle() {
			return angle;
		}
		private void setAngle(double angle) {
			this.angle = angle;
		}
		
		
		
	}
}
